"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var UglifyJS = require("uglify-js");
var esprima = require("esprima");
var escope = require('escope');
function hasUndefined(globalReferenceNames) {
    return globalReferenceNames.indexOf('undefined') !== -1;
}
function withoutUndefined(globalReferenceNames) {
    return globalReferenceNames.filter(function (globalReferenceName) { return globalReferenceName !== 'undefined'; });
}
function getIIFEBody(code, globalReferenceNames) {
    if (hasUndefined(globalReferenceNames)) {
        return "(function(undefined," + withoutUndefined(globalReferenceNames).join(',') + "){" + code + "})";
    }
    return "(function(" + globalReferenceNames.join(',') + "){" + code + "})";
}
function getImplicitlySafeCodeInvocation(globalReferenceNames) {
    if (hasUndefined(globalReferenceNames)) {
        return ".apply(this,void 0,\"" + withoutUndefined(globalReferenceNames).join(',') + "\".split(\",\").map(function(k){return this[k]}))";
    }
    return ".apply(this,\"" + globalReferenceNames.join(',') + "\".split(\",\").map(function(k){return this[k]}))";
}
function getExplicitlySafeCodeInvocation(globalReferenceNames) {
    if (hasUndefined(globalReferenceNames)) {
        return "(void 0," + withoutUndefined(globalReferenceNames).map(function (globalReferenceName) { return "this." + globalReferenceName; }).join(',') + ")";
    }
    return "(" + globalReferenceNames.map(function (globalReferenceName) { return "this." + globalReferenceName; }).join(',') + ")";
}
function getWrappedCode(code, globalReferenceNames) {
    var iifeBody = getIIFEBody(code, globalReferenceNames);
    var implicitlySafeCodeInvocation = getImplicitlySafeCodeInvocation(globalReferenceNames);
    var explicitlySafeCodeInvocation = getExplicitlySafeCodeInvocation(globalReferenceNames);
    /* istanbul ignore next */
    if (implicitlySafeCodeInvocation.length < explicitlySafeCodeInvocation.length) {
        return iifeBody + implicitlySafeCodeInvocation;
    }
    return iifeBody + explicitlySafeCodeInvocation;
}
function squeeze(code, _a) {
    var _b = _a === void 0 ? {} : _a, _c = _b.minifyOptions, minifyOptions = _c === void 0 ? {} : _c, _d = _b.mangleReadwriteVariables, mangleReadwriteVariables = _d === void 0 ? false : _d, _e = _b.excludedNames, excludedNames = _e === void 0 ? [] : _e, _f = _b.minRepeatCount, minRepeatCount = _f === void 0 ? 2 : _f, _g = _b.minNameLength, minNameLength = _g === void 0 ? 2 : _g;
    try {
        var ast = esprima.parseScript(code);
        var scopeManager = escope.analyze(ast);
        var globalScope_1 = scopeManager.globalScope;
        var globalReferenceDictionary_1 = {};
        var oftenUsedGlobalReferenceNames_1 = [];
        globalScope_1.implicit.left.forEach(function (reference) {
            var identifier = reference.identifier;
            var name = identifier.name;
            if (!mangleReadwriteVariables) {
                if (globalScope_1.implicit.variables.findIndex(function (variable) {
                    return variable.name === name;
                }) !== -1) {
                    return;
                }
            }
            if (name.length >= minNameLength) {
                if (globalReferenceDictionary_1[name]) {
                    globalReferenceDictionary_1[name].push(reference);
                }
                else {
                    globalReferenceDictionary_1[name] = [reference];
                }
            }
        });
        Object
            .keys(globalReferenceDictionary_1)
            .forEach(function (referenceName) {
            var references = globalReferenceDictionary_1[referenceName];
            if (references.length >= minRepeatCount && excludedNames.indexOf(referenceName) === -1) {
                oftenUsedGlobalReferenceNames_1.push(referenceName);
            }
        });
        var wrappedCode = getWrappedCode(code, oftenUsedGlobalReferenceNames_1);
        var output = UglifyJS.minify(wrappedCode, minifyOptions);
        /* istanbul ignore next */
        if (output.error) {
            return {
                minifyOutput: output,
                error: output.error
            };
        }
        return {
            code: output.code,
            minifyOutput: output,
            mangledGlobalReferenceNames: oftenUsedGlobalReferenceNames_1
        };
    }
    catch (e) {
        return {
            error: e
        };
    }
}
exports.squeeze = squeeze;
//# sourceMappingURL=js-juicer.js.map